/**************************************************************************   
 *                                                                        *
 *    File Name : Inverse_master.c                                        *
 *                                                                        *
 *    Author    : KIM, DONGMIN (kimd@top.cis.syr.edu)                     *
 *                                                                        *
 *    Function  : This master program takes NxN matrix and                *
 *                decomposes this matrix into L and U matrices.           *
 *                                                                        *
 **************************************************************************/

#include "p4.h"
#include <stdio.h>
/*
#include <stdlib.h>
*/
#include <math.h>

/* FOR VDCE
 *
 */
#include "dvm.h"
#include "inet.h"
#include <memory.h>
#include <sys/param.h>

#define	DATA	1
#define	ZERO	0
#define L       1
#define U       2

float **MATRIX;

/**************************************************************************   
 *								          *
 *	PROCEDURE	: display()				          *
 *	INPUT		: int					          *
 *	FUNCTION	: Display the LU matrix.		          *
 *								          *
 **************************************************************************/
display(size)
int	size;
{
  int     i, j;
  for ( i = 0 ; i < size ; i++ ) {
    for ( j = 0 ; j < size ; j++ )
      printf(" %10.5f ", MATRIX[i][j] );
    printf("\n");
  }
}

/**************************************************************************   
 *								          *
 *	PROCEDURE	: main					          *
 *								          *
 **************************************************************************/
main(argc, argv)
int     argc;
char    **argv;
{
  FILE   *fp;
  int    i, j, k, l, matrix_size, check_eof, LU;
  int	 to, type, int_size, from, real_size, size;
  int	 load, nslave, row, column;
  int    Int_size, Real_size;
  float *incoming;
  float	 element;
  long   total_elapsed;

  /*
   * For VDCE
  */

  struct task *dvminfo;
  int  no_of_data;
  int  no_of_argc;
  char *arglist[100];
  int  mytaskid;
  int  cvmsockport;


  if (argc < 4) {
    printf("Usage: (1) input file name  (A matrix)\n");
    printf("Usage: (2) output file name (Inverse matrix)\n");
    printf("Usage: (3) output file name (timer info)\n");
    exit(1);
  }

  timer_reset();
  
  p4_initenv(&argc, argv);
  p4_create_procgroup();
  if(p4_get_my_id() != 0){
      slave();
	  p4_wait_for_end();
	  exit(0);
  }

 
  /*
   * Add VDCE Init and receiving data
   */

   cvmsockport = atoi(argv[1]);
   mytaskid    = atoi(argv[2]);
   no_of_data = atoi(argv[3]);

   printf("vdce init start\n");

   arglist[0] = (char *) calloc(128,sizeof(char));
   sprintf(arglist[0],"lu");

    /*
     * allocation memory for task information structure
     */
    if ( (dvminfo = (struct task *) malloc(sizeof(struct task)*no_of_data)) ==NULL)
                err_dump("vdce_init: No memory");

   vdce_init(cvmsockport,dvminfo,mytaskid,no_of_data,&no_of_argc,arglist);
   printf("done vdce_init\n");

   vdce_recv(mytaskid,&no_of_argc,arglist,dvminfo,no_of_data);

   printf("done vdce_recv\n");

     { int p,q;
        q=1;
#ifdef DEBUG
        fprintf(stderr,"no_of_argc = %d\n",no_of_argc);
#endif
        for(p=1;p<no_of_argc;p++) {
           if (argc > no_of_argc)
                strcpy(argv[p],arglist[p]);
           else {
             argv[p] = (char *) calloc(MAXPATHLEN,sizeof(char));
             strcpy(argv[p],arglist[p]);
           }
#ifdef DEBUG
        fprintf(stderr,"vdce_main: argv[%d]=%s\n",p,argv[p]);
#endif
       }
     }

  nslave	= p4_num_total_ids() - 1;

  Int_size  = sizeof(int);
  Real_size = sizeof(float);

  /* get input data from file */
  fp = fopen(argv[1], "r");

  if ( fp == NULL )
    printf("\ncannot find file!\n");

  /* read the size of the matrix. */
  fscanf( fp, "%d", &matrix_size );

  /* read the size of the matrix. */
  fscanf( fp, "%d", &LU );

/*
  printf("# of slave : %d\n", nslave);
  printf("Type of the matrix : ");
  if ( LU == L )
    printf("L\n");
  else
    printf("U\n");

  printf("# of matrix_size  : %d\n", matrix_size);
*/

  /* set the array MATRIX using the matrix size */
  MATRIX = (float **) calloc(matrix_size, sizeof(float *));
  for ( i = 0 ; i < matrix_size ; i++ )
    MATRIX[i] = (float *) calloc(matrix_size, Real_size);

  /* initialize MATRIX with value 0. */
  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      MATRIX[i][j] = 0.;
    }
  }

  /* read the elements of the matrix. */
  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      check_eof = fscanf( fp, "%f", &element);
      if ( check_eof == EOF )
	break;
      else
	MATRIX[i][j] = element;
    }
  }
  fclose(fp);

/*
  printf("\n\n********************************\n");
  printf("* MASTER : The Original Matrix *\n");
  printf("********************************\n\n");
  display(matrix_size);
*/

  load	= matrix_size / nslave;

  /* distribute data to each slave */
  /* broadcast the number of rows, size of matrix and MATRIX */

  p4_broadcast(DATA, &load, Int_size);
  p4_broadcast(DATA, &LU, Int_size);
  p4_broadcast(DATA, &matrix_size, Int_size);

  size = matrix_size * Real_size;
  for ( row = 0 ; row < matrix_size ; row++) {
    p4_broadcast(DATA, MATRIX[row], size);
  }

  /* receive all data from each slave */
  type = DATA;

  /* receive all data from each slave */
  for ( from = 1 ; from <= nslave ; from++ ) {
    column = (from - 1) * load;
    /* receive the data */
    for ( i = 0 ; i < load ; i++) {
      /* receive the data by column */
      incoming	= NULL;
      p4_recv(&type, &from, &incoming, &size);
      for ( j = 0 ; j < matrix_size ; j++ ) {
	MATRIX[j][column + i] = incoming[j];
      }
      p4_msg_free(incoming);
    }
  }

  
  total_elapsed = timer_elapsed();

  printf("\n\n****************************************\n");
  printf("* Inverse matrix of ");

  if ( LU == L )
    printf("L\n");
  else
    printf("U\n");

  printf("*\n");
  printf("* total elapsed time\n");
  printf("* %d (milliseconds)\n", total_elapsed);
  printf("****************************************\n");
  /*fp = fopen("../DATA/t3.dat","w");
	fprintf(fp,"%d",total_elapsed);
  fclose(fp);*/


fflush(stdout);

  p4_wait_for_end();

/*
  printf("\n\n***********************\n");
  printf("* MASTER : The result *\n");
  printf("***********************\n\n");
  display(matrix_size);
*/

  /* make the result to the file */
  fp = fopen(argv[2], "w");
  fprintf( fp, "%d\n", matrix_size);
  if ( fp == NULL ) {
    printf("can't open file %s\n", argv[2]);
    exit(1);
  }

  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      fprintf( fp, "%6.3f ", MATRIX[i][j]);
    }
    fprintf( fp, "\n");
  }
  fclose(fp);
/*
  fp = fopen(argv[3],"w");
	fprintf( fp, "%d\n", total_elapsed);
  fclose(fp);
*/


  /*
   * Add VDCE Communication
  */
  vdce_send(mytaskid,no_of_argc,arglist,dvminfo,no_of_data);
  vdce_end();

} /* End of Master */
