/**************************************************************************   
 *                                                                        *
 *    File Name : Inverse_slave.c                                         *
 *                                                                        *
 *    Author    : KIM, DONGMIN (kimd@top.cis.syr.edu)                     *
 *                                                                        *
 *    Function  : This slave program takes some rows from master program, *
 *                and computes the inverse of the matrix.                 *
 *                                                                        *
 **************************************************************************/

#include "p4.h"
#include <stdio.h>
/*
#include <stdlib.h>
*/
#include <math.h>

#define	MASTER	0
#define DATA    1
#define DATA2   10
#define L       1
#define U       2

slave() {
  int    int_size, real_size, master, my_id, size;
  int	 i, j, k, step, type, from, to;
  int	 matrix_size, load, LU;
  int	 current_id, current_row, nslave;
  int	 total_size;
  int	 *incoming1;
  float *incoming2;
  float **I, **inverse, **MATRIX;
  float divisor, sum;
  int	 current, current_slave;
  
  int_size  = sizeof(int);
  real_size = sizeof(float);
  
  master = 0;
  my_id	= p4_get_my_id();
  nslave = p4_num_total_ids() - 1;
  
  /* receive load, matrix size and data from MASTER */
  type = DATA;
  from = master;

  /* receive the "load" from MASTER */
  incoming1 = NULL;
  p4_recv(&type, &from, &incoming1, &int_size);
  load = *incoming1;
  p4_msg_free(incoming1);

  /* receive the type of the matrix from MASTER */
  incoming1 = NULL;
  p4_recv(&type, &from, &incoming1, &int_size);
  LU = *incoming1;
  p4_msg_free(incoming1);

  /* receive the "matrix size" from MASTER */
  incoming1 = NULL;
  p4_recv(&type, &from, &incoming1, &int_size);
  matrix_size = *incoming1;
  p4_msg_free(incoming1);

  /* set the slave's array.
     I       : identity matrix.
     MATRIX  : the original matrix.
     inverse : the inverse matrix.
  */
  I = (float **) calloc(matrix_size, sizeof(float *));
  for ( i = 0 ; i < matrix_size ; i++ )
    I[i] = (float *) calloc(load, sizeof(float));

  MATRIX = (float **) calloc(matrix_size, sizeof(float *));
  for ( i = 0 ; i < matrix_size ; i++ )
    MATRIX[i] = (float *) calloc(matrix_size, sizeof(float));

  inverse = (float **) calloc(load, sizeof(float *));
  for ( i = 0 ; i < load ; i++ )
    inverse[i] = (float *) calloc(matrix_size, sizeof(float));

  /* receive the data from MASTER */
  size	= matrix_size * real_size;
  for ( i = 0 ; i < matrix_size ; i++ ) {
    incoming2 = NULL;
    p4_recv(&type, &from, &incoming2, &size);
    for ( j = 0 ; j < matrix_size ; j++ ) {
      MATRIX[i][j] = incoming2[j];
    }
    p4_msg_free(incoming2);
  }

  /* setup the indentity matrix, I. */
  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < load ; j++ ) {
      if ( i == ((my_id - 1) * load + j) ) {
	I[i][j] = 1.;
      }
      else {
	I[i][j] = 0.;
      }
    }
  }

/*
  printf("\n=======================\n");
  printf("Slave : %d\n\n", my_id);
  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      printf(" %5.2f ", MATRIX[i][j]);
    }
    printf("\n");
  }
  printf("\n\n\n");

  for ( i = 0 ; i < matrix_size ; i++ ) {
    for ( j = 0 ; j < load ; j++ ) {
      printf(" %5.2f ", I[i][j]);
    }
    printf("\n");
  }
*/

  /* Warning :
     the inverse matrix of L (or U) is also L (or U).
     So we can reduce the computation time if we notice this.
  */
  if ( LU == L ) { /* lower triangular matrix */

    for ( k = 0 ; k < load ; k++ ) {
      for ( i = 0 ; i < matrix_size ; i++ ) {
	sum = 0.;
	for ( j = 0 ; j < i ; j++ ) {
	  sum = sum + MATRIX[i][j] * inverse[k][j];
	}
	inverse[k][i] = I[i][k] - sum;
      }
    }
  }
  else { /* upper triangular matrix */

    for ( k = 0 ; k < load ; k++ ) {
      for ( i = matrix_size - 1 ; i >= 0 ; i-- ) {
	divisor = MATRIX[i][i];
	sum = 0.;
	for ( j = i + 1 ; j < matrix_size ; j++ ) {
	  sum = sum + MATRIX[i][j] * inverse[k][j];
	}
	inverse[k][i] = ( I[i][k] - sum ) / divisor;
      }
    }
  }

/*
  printf("\n Result \n");
  for ( i = 0 ; i < load ; i++ ) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      printf(" %5.2f ", inverse[i][j]);
    }
  }
  printf("\n=======================\n");
*/

  /* send the result matrix to the MASTER */
  size	= matrix_size * real_size;
  for ( i = 0 ; i < load ; i++ ) {
    p4_send(DATA, MASTER, inverse[i], size);
  }
  free(I);
  free(MATRIX);
  free(inverse);
}


/*
main (argc, argv)
     int     argc;
     char    **argv;
{
  p4_initenv(&argc, argv);
  p4_create_procgroup();
  slave();
  p4_wait_for_end();
  exit(0);
}*/ /* End of Slave */
