/*
 * Read a stream socket one line at a time, and write each line back
 * to the sender.
 *
 * Return when the connection is terminated.
 */

#include        <stdio.h>
#define MAXLINE 512

str_echo(sockfd)
int     sockfd;
{
        int     n;
        char    line[MAXLINE];

        for ( ; ; ) {
                n = readline(sockfd, line, MAXLINE);
                if (n == 0)
                        return;         /* connection terminated */
                else if (n < 0)
                        err_dump("str_echo: readline error");

                if (writen(sockfd, line, n) != n)
                        err_dump("str_echo: writen error");
        }
}
/*
 * Read the contents of the FILE *fp, write each line to the
 * stream socket (to the server process), then read a line back from
 * the socket and write it to the standard output.
 *
 * Return to caller when an EOF is encountered on the input file.
 */

str_cli(fp, sockfd)
register FILE   *fp;
register int    sockfd;
{
        int     n;
        char    sendline[MAXLINE], recvline[MAXLINE + 1];

        while (fgets(sendline, MAXLINE, fp) != NULL) {
                n = strlen(sendline);
                if (writen(sockfd, sendline, n) != n)
                        err_sys("str_cli: writen error on socket");

                /*
                 * Now read a line from the socket and write it to
                 * our standard output.
                 */

                n = readline(sockfd, recvline, MAXLINE);
                if (n < 0)
                        err_dump("str_cli: readline error");
                recvline[n] = 0;        /* null terminate */
                fputs(recvline, stdout);
        }

        if (ferror(fp))
                err_sys("str_cli: error reading file");
}

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for the newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
        int     n, rc;
        char    c;

        for (n = 1; n < maxlen; n++) {
                if ( (rc = read(fd, &c, 1)) == 1) {
                        *ptr++ = c;
                        if (c == '\n')
                                break;
                } else if (rc == 0) {
                        if (n == 1)
                                return(0);      /* EOF, no data read */
                        else
                                break;          /* EOF, some data was read */
                } else
                        return(-1);     /* error */
        }

        *ptr = 0;
        return(n);
}

/*
 * Write "n" bytes to a descriptor.
 * Use in place of write() when fd is a stream socket.
 */

int
writen(fd, ptr, nbytes)
register int    fd;
register char   *ptr;
register int    nbytes;
{
        int     nleft, nwritten;

        nleft = nbytes;
        while (nleft > 0) {
                nwritten = write(fd, ptr, nleft);
                if (nwritten <= 0)
                        return(nwritten);               /* error */

                nleft -= nwritten;
                ptr   += nwritten;
        }
        return(nbytes - nleft);
}

