/*
 * File Name : util.c
 *
 * Description : This file contains utility functions as followings:
 *               err_dump(),
 *               err_sys(),
 *	 	 err_ret(),
 *		 gethostaddr(),
 *		 pri_inet(),
 *		 getfilename()
 *
 * Programmed by : IlKyeun Ra, Syracuse University
 *           
*/
  
#include        <stdio.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <netinet/in.h>
#include        <arpa/inet.h>
#include	<netdb.h>
#include        <sys/param.h>

err_dump(msg)
char *msg;
{
	fprintf(stderr,"%s \n",msg);
        fflush(stderr);
	abort();	/* ask core dump */
}

err_sys(msg)
char *msg;
{
        fprintf(stderr,"%s \n",msg);
        fflush(stderr);
        exit(1);
}

err_ret(msg)
char *msg;
{
        fprintf(stderr,"%s \n",msg);
        fflush(stderr);
        return(NULL);
}

/*
 * Function Name : gethostaddr()
 *
 * Description   : get internet address from hostname
 *
 * Input(s)      : None
 *
 * Output(s)     : Internet address(e.g. 128.230.32.2)
*/
char *gethostaddr()
{
  char *hname;
  char *haddr;
  int  hlen=100;
  struct hostent *he;

  hname = (char *) calloc(hlen,sizeof(char));
  if (gethostname(hname,hlen)!=0) 
      err_sys("VDCE_INIT: can't find host name");

#ifdef DEBUG
/*
  printf("hostname = %s\n",hname);
*/
#endif

  if ( (he = gethostbyname(hname)) == NULL)
      err_sys("VDCE_INIT: can`t get host address");

  if ( (haddr = (char *) calloc(hlen,sizeof(char))) == NULL)
	err_dump("VDCE_INIT: No more memory");

  switch ( he->h_addrtype) {
    case AF_INET:
       pr_inet(he->h_addr_list,he->h_length,haddr);
       break;
    default:
	err_ret("unknow address type");
        break;
  }
  
#ifdef DEBUG
/*
  printf("my host name and address : %s %s\n",hname,haddr);
*/
#endif
  return(haddr);
}
/*
 * Function Name : pr_inet()
 *
 * Description   : convert internet address to ascii format(user readable)
 *
 * Input(s)      : internet - not user readable format
 *                 length   - length of internet address
 * 
 * Output(s)     : User readable internet address - ascii format
*/
pr_inet(listptr,length,addr)
char **listptr;
int  length;
char *addr;
{

     struct in_addr *ptr;

     while ( (ptr = (struct in_addr *) *listptr++) != NULL) 
       sprintf(addr,"%s",inet_ntoa(*ptr));
}

/*
 * Function name : getfilename()
 *
 * Description   : This function is creating a temporary file name 
 *
 * Result        : 1 - Success, 0 - Error
 *                 
*/
getfilename(fname)
char *fname;
{
     char pathname[MAXPATHLEN];
   
     if (getwd(pathname) != NULL) 
         if ( (fname = tempnam(&pathname[0],"vdce")) != NULL)
	   return(1);
         else 
	   fprintf(stderr,"VDCE: can't creat a tempoary file\n");
     else
        fprintf(stderr,"VDCE: can't get current directory\n");

     return(0);
}
