/*
 * File Name : vdce_recv.c
 *
 * Description : This file contains some functions related to the VDCE_RECV function 
 *               which receives data from parent task via socket.
 *
 * Fuunctions  : vdce_recv()
 *               get_data()
 *
 * Programmed by : Ilkyeun Ra, Syracuse University
 *
 * Date        : December 15, 1996
 *
 * Update History
 *
 */
#include "inet.h"
#include "dvm.h"

#define External	-1 
#define Remote           0
#define Local            1

/*
 * Function Name : vdce_recv
 *
 * Description   : Receiving all data associated with my task from parent
 *                 task via a socket
 *
 * Input(s)      : mytaskid - my task id
 *                 argc     - no. of argument of my task
 *                 argv     - argument list of my task
 *                 dvminfo  - task execution environment information
 *                 no_of_data - no. of data 
 *
 * Output(s)     : Receiving data using a socket
 *
*/

vdce_recv(mytaskid,argc,argv,dvminfo,no_of_data)
int      mytaskid;
int      *argc;
char     *argv[];
struct   task *dvminfo;
int      no_of_data;
{
    FILE	*fd;
    char	*msg;
    char	*path;
    int         i, no_of_inputs, no_of_outputs;
    int	        n=1;
    	
   /* for error message */ 
    msg = (char *) calloc(MAXPATHLEN,sizeof(char));

    /*
     * Find the related input and out data 
    */
    for (i=0;i<no_of_data;i++) {
        /*
	 * check data that are related to my task
	 */
        if (dvminfo[i].d_tkid == mytaskid && dvminfo[i].d_pid !=-1) { 
           /*
	    * Check type of input data whether data is external or internal
	    */
	   if (dvminfo[i].s_pid == External) {
	      if ( (fd = fopen(dvminfo[i].v_name,"r")) == NULL) {
		  sprintf(msg,"VDCE_RECV: can't open input file %s\n",
		          dvminfo[i].v_name);
	          err_ret(msg);
	      }
	      else {
	           /* 
	      	    * Provide input file name 
		    */
/*
		    strcpy(argv[n],dvminfo[i].v_name);
		    n++; 
*/
		    fclose(fd);
	      } /* end of data is located at local */
           } /* end of external */
	   else {   
	        /* 
       	         * input data is located at remote 
	         * task should receive data now
                */
	        if (get_data(&dvminfo[i],argv[n])==0) {
	           sprintf(msg,"VDCE_RECV: can't receive data %s\n",
	      	           dvminfo[i].v_name);
		   err_ret(msg);
	        }
	   } /* end of data is located at remote */
        } /* done with my task id */
        else {
	  /*
	   * none of my business
	   */
	}
     } /* for end */
#ifdef DEBUG
     fprintf(stderr,"no_of_argc = %d\n",*argc);
     for (i=i;i<*argc;i++) fprintf(stderr,"vdce_debug: argv[%d]=%s\n",i,argv[i]);
#endif
}

/*
 * Function Name : Get_data()
 *
 * Description   : Get the data from parent task by socket 
 *
 * Input(s)      : taskinfo - task execution environment information
 *                 argv     - an external file name or vairiable name
 *
 * Output(s)     : receiving data via a socket
 *
 *
*/
get_data(taskinfo,argv)
struct task *taskinfo;
char        *argv;
{
  int result ;
  int portnum, sockfd;

  struct sockaddr_in src_addr;
  int optval = 1;

  int is_connect = 0;
  int time_out   = 0;

#ifdef DEBUG
  fprintf(stderr,"vdce_recv: get_data starts\n");
#endif

  /*
   * sending data with respect to data types
  */
  switch (taskinfo->v_type) {
   case File:  /* file type */
    {
       if ( getfilename(argv) == 0 ) {
	    err_sys("VDCE_RECV: can't create temporay file ");
       }

/*
       setup_recv_path(taskinfo);
*/

       /*
        * receive data from previous task through socket
        */
       result=receive_data_to_file(taskinfo->d_socket,argv);
       if (result==0) {
          err_sys("VDCE_RECV: can't receive data via socket");
          close(sockfd);
       } 
       break;
    } /* end case 0 */

    case Integer   : /* interger data type */
    case Char      : /* char data type */
    case Float     : /* float data type */
    case Structure : /* struct data type */
    default: break;

  } /* switch end */

  return result; /* return result */
} /* end of get_data */

