/*
 * File Name     : vdce_send.c
 *
 * Function Name : vdce_send
 *
 * Description   : send data to children tasks through socket
 *
 * Input(s)      : mytaskid - source task id
 *                 no_of_argc - 
 *                 arglist -
 *                 taskinfo - data routing information
 *                 no_of_data - no of data that are involving this application
 *
 * Output(s)     : sending data to specified tasks
 *
 *  Programmed by : Ilkyeun Ra , Syracuse University
 *
*/

#include "dvm.h"
#include "inet.h"

vdce_send(mytaskid, argc, argv, taskinfo,no_of_data)
int         mytaskid;
int         argc;
char        **argv;
struct task *taskinfo;
{

  int i,n;
  int sockfd;

  for (i=0; i < no_of_data; i++) {
      /*
       * Check data needed to send
       */
      if ( taskinfo[i].s_tkid == mytaskid && taskinfo[i].s_pid != External &&
           taskinfo[i].d_pid != -1) {
         /*
          * This data may be sent to a task, let's find destination tasks
         */
          if (send_data_to_task(&taskinfo[i]) == 0) {
	      fprintf(stderr,"VDCE_SEND: can't send data %s to %s\n",
		      taskinfo[i].v_name,taskinfo[i].d_name);
	      return;
	  }
	  /*
           * This case will be modified upon changing input type, but
           * I assume that user input/outputs are files.
           * Temporary files are removed
          */
      }
  }
  
} /* end vdec_send */

/*
 * Function Name : send_data_to_task
 *
 * Description   : Sending data via socket
 *
 * Input(s)      : information of task execution enviroment
 *
 *
*/
send_data_to_task(taskinfo)
struct task *taskinfo;
{
     int result;

     /*
      * Open local result and send the result to other task via socket
     */
     switch (taskinfo->v_type) {
      case File : {
/*
                     setup_sending_path(taskinfo);
*/
		     result = send_data_from_file(taskinfo->s_socket,taskinfo->v_name);
#ifdef DEBUG                     
                fprintf(stderr,"vdce_send: %s is sent to task %d from % d\n",taskinfo->v_name,
                        taskinfo->d_tkid,taskinfo->s_tkid);                 
#endif

		     break;
      } 
      case Integer :
      case Char     :
      case Float    :
      case Structure:
      default       : break;
     } /* end switch */
      return result;
} /* end send_data_to_task */

